const fs = require('fs');
const path = require('path');

// Directory for node_modules
const nodeModulesDir = path.resolve(__dirname, 'node_modules');
const importmap = { imports: {} };

// Function to resolve `exports` in `package.json
function resolveExports(moduleName, modulePath) {
    const packageJsonPath = path.join(modulePath, 'package.json');
    if (!fs.existsSync(packageJsonPath)) return;

    const packageJson = JSON.parse(fs.readFileSync(packageJsonPath, 'utf-8'));

    // If `exports` is defined, process the entries
    if (packageJson.exports) {
        Object.entries(packageJson.exports).forEach(([key, exportValue]) => {
            if (key.includes('*')) {
                handleWildcardExport(moduleName, modulePath, key, exportValue);
            } else {
                const resolvedPath = resolveNestedExportPath(modulePath, exportValue);
                if (resolvedPath && (resolvedPath.endsWith('.mjs') || resolvedPath.endsWith('.js'))) {
                    const importKey = path.join(moduleName, key).replace(/\\/g, '/').replace(/^\.\//, '');
                    importmap.imports[importKey] = `./${resolvedPath}`;
                }
            }
        });
    } else {
        // Use fallbacks for `main`, `module`, or `browser` if they are defined as string and `.mjs`/`.js`.
        const fallbackPath = (typeof packageJson.module === 'string' && (packageJson.module.endsWith('.mjs') || packageJson.module.endsWith('.js')))
            ? packageJson.module
            : (typeof packageJson.main === 'string' && (packageJson.main.endsWith('.mjs') || packageJson.main.endsWith('.js')))
                ? packageJson.main
                : (typeof packageJson.browser === 'string' && (packageJson.browser.endsWith('.mjs') || packageJson.browser.endsWith('.js')))
                    ? packageJson.browser
                    : null;

        if (fallbackPath) {
            const mainPath = path.join(modulePath, fallbackPath);
            if (fs.existsSync(mainPath)) {
                importmap.imports[moduleName] = `./${path.relative(__dirname, mainPath).replace(/\\/g, '/')}`;
            }
        }
    }

    // Look further into subdirectories recursively
    fs.readdirSync(modulePath, { withFileTypes: true }).forEach(file => {
        if (file.isDirectory()) {
            const submoduleName = `${moduleName}/${file.name}`;
            const submodulePath = path.join(modulePath, file.name);
            resolveExports(submoduleName, submodulePath); // Rekursiver Aufruf
        }
    });
}

// Function for resolving `exports` structures
function resolveNestedExportPath(modulePath, exportValue) {
    if (typeof exportValue === 'string') {
        return path.relative(__dirname, path.join(modulePath, exportValue)).replace(/\\/g, '/');
    } else if (exportValue && typeof exportValue === 'object') {
        // Suche zuerst nach `import`-Pfad im `.mjs`/`.js`-Format, dann `require`, dann `default`
        const subPath = exportValue.import?.default || exportValue.import || exportValue.default || exportValue.require;
        return typeof subPath === 'string' && (subPath.endsWith('.mjs') || subPath.endsWith('.js'))
            ? path.relative(__dirname, path.join(modulePath, subPath)).replace(/\\/g, '/')
            : null;
    }
    return null;
}

// Function for processing wildcard exports
function handleWildcardExport(moduleName, modulePath, key, exportValue) {
    const [baseDir] = key.split('/*');
    const fullBaseDir = path.join(modulePath, baseDir);

    if (!fs.existsSync(fullBaseDir)) return;

    const files = fs.readdirSync(fullBaseDir, { withFileTypes: true });
    files.forEach(file => {
        if (file.isDirectory()) {
            const submodulePath = path.join(fullBaseDir, file.name);
            const resolvedPath = resolveNestedExportPath(fullBaseDir, exportValue);
            if (resolvedPath && (resolvedPath.endsWith('.mjs') || resolvedPath.endsWith('.js'))) {
                const importKey = `${moduleName}/${file.name}`;
                importmap.imports[importKey] = `./${resolvedPath.replace('*', file.name)}`;
            }
        }
    });
}

// Main function for creating the import map
function generateImportMap() {
    fs.readdirSync(nodeModulesDir).forEach(module => {
        const modulePath = path.join(nodeModulesDir, module);

        if (module.startsWith('.') || !fs.lstatSync(modulePath).isDirectory()) return;

        if (module.startsWith('@')) {
            fs.readdirSync(modulePath).forEach(scopedModule => {
                const fullModuleName = `${module}/${scopedModule}`;
                const scopedModulePath = path.join(modulePath, scopedModule);
                resolveExports(fullModuleName, scopedModulePath);
            });
        } else {
            resolveExports(module, modulePath);
        }
    });

    // Punkte aus den Pfaden entfernen
    for (const key in importmap.imports) {
        importmap.imports[key] = importmap.imports[key].replace(/^\.\/node_modules/, '/node_modules');
    }

    // Saving the import map
    fs.writeFileSync(
        path.resolve(__dirname, 'importmap.json'),
        JSON.stringify(importmap, null, 2),
        'utf-8'
    );

    console.log("Import map was successfully created and saved as 'importmap.json'.\n");
}

generateImportMap();
